/**
 *	@file	GetPixelsImageSettingsContainer.h
 *	@author	ENDO
 */

#ifndef GETPIXELSIMAGESETTINGSCONTAINER_H
#define GETPIXELSIMAGESETTINGSCONTAINER_H

#include <aknsettingitemlist.h>	// for CAknSettingItemList.
#include <akntabobserver.h>		// for MAknTabObserver.

#include "GetPixels.hrh"		// for ESettingsImageTabNum, KMaxJpegCommentLength.

class TGetPixelsImageSettings
	{
public:
	inline TGetPixelsImageSettings(void) :
		iPngColors(EImageSettingsPngColors16M),
		iPngCompressLevel(EImageSettingsPngCompressLevelDefaultCompression),
		iBmpColors(EImageSettingsBmpColors16M),
		iMbmColors(EImageSettingsMbmColorsDefault),
		iJpegQuality(100),
		iJpegColorSampling(EImageSettingsJpegColorSampling444)
		{
		};

public:
	TInt iPngColors;
	TInt iPngCompressLevel;

	TInt iBmpColors;

	TInt iMbmColors;

	TInt iJpegQuality;
	TInt iJpegColorSampling;
	TBuf<KMaxJpegCommentLength> iJpegComment;
	};

class MGetPixelsImageSettingItemListObserver
	{
public:
	virtual void NotifyEditItemL(TInt aIndex, TBool aCalledFromMenu) = 0;
	};

class CAknSettingItem;
class CGetPixelsImageSettingItemList : public CAknSettingItemList
	{
public:
	CGetPixelsImageSettingItemList(MGetPixelsImageSettingItemListObserver& aObserver, TGetPixelsImageSettings& aSettings);

public:
	// from CAknSettingItemList.
	CAknSettingItem* CreateSettingItemL(TInt aIdentifier);
	void EditItemL(TInt aIndex, TBool aCalledFromMenu);

private:
	// from CAknSettingItemList.
	void SizeChanged(void);

private:
	MGetPixelsImageSettingItemListObserver& iObserver;
	TGetPixelsImageSettings& iSettings;
	};

class CAknNavigationDecorator;
class CAknTabGroup;
class CGetPixelsSettings;
class CGetPixelsAppUi;
class CGetPixelsImageSettingsContainer : public CCoeControl, public MAknTabObserver, public MGetPixelsImageSettingItemListObserver
	{
public:
	static CGetPixelsImageSettingsContainer* NewL(const TRect& aRect);
	virtual ~CGetPixelsImageSettingsContainer();

private:
	CGetPixelsImageSettingsContainer(void);
	void ConstructL(const TRect& aRect);

public:
	void SetActiveTab(TInt aIndex);
	void ChangeSelectedItemL(void);
	void ReadSettingsFromModelL(const CGetPixelsSettings& aModel);
	void WriteSettingsToModelL(CGetPixelsSettings& aModel) const;

private:
	// from CCoeControl.
	TInt CountComponentControls(void) const;
	CCoeControl* ComponentControl(TInt aIndex) const;
	TKeyResponse OfferKeyEventL(const TKeyEvent& aKeyEvent, TEventCode aType);

private:
	// from MAknTabObserver.
	void TabChangedL(TInt aIndex);

public:
	// from MGetPixelsSettingItemListObserver.
	void NotifyEditItemL(TInt aIndex, TBool aCalledFromMenu);

private:
	TFixedArray<CGetPixelsImageSettingItemList*, ESettingsImageTabNum> iSettingItemLists;
	TInt iCurrentTabIndex;
	CAknNavigationDecorator* iNavigationDecorator;
	CAknTabGroup* iTabGroup;
	TGetPixelsImageSettings iSettings;
	CGetPixelsAppUi& iAppUi;
	};

#endif // GETPIXELSIMAGESETTINGSCONTAINER_H
