/**
 *	@file	GetPixelsSettings.h
 *	@author	ENDO
 */

#ifndef GETPIXELSSETTINGS_H
#define GETPIXELSSETTINGS_H

#include <imageconversion.h>

class CGetPixelsSettings : CBase
	{
public:
	static CGetPixelsSettings* NewL(void);
	static CGetPixelsSettings* NewLC(void);
	virtual ~CGetPixelsSettings();

private:
	CGetPixelsSettings(void);
	void ConstructL(void);

public:
	void ExternalizeL(RWriteStream& aStream) const;
	void InternalizeL(RReadStream& aStream);

public:
	inline TInt SetImageFormat(const TUid& aImageFormat)
		{
		iImageFormat = aImageFormat;
		return KErrNone;
		};

	inline TInt SetEnableThreading(TBool aEnableThreading)
		{
		iEnableThreading = aEnableThreading;
		return KErrNone;
		};

	inline TInt SetEnableSaveToExternal(TBool aEnableSaveToExternal)
		{
		iEnableSaveToExternal = aEnableSaveToExternal;
		return KErrNone;
		};

	inline TInt SetSavedDirectoryName(const TDesC& aSavedDirectoryName)
		{
		HBufC* tmpSavedDirectoryName = aSavedDirectoryName.Alloc();
		if (tmpSavedDirectoryName == NULL)
			{
			return KErrNoMemory;
			}
		delete iSavedDirectoryName;
		iSavedDirectoryName = tmpSavedDirectoryName;
		return KErrNone;
		};

	inline TInt SetSavedFileName(const TDesC& aSavedFileName)
		{
		HBufC* tmpSavedFileName = aSavedFileName.Alloc();
		if (tmpSavedFileName == NULL)
			{
			return KErrNoMemory;
			}
		delete iSavedFileName;
		iSavedFileName = tmpSavedFileName;
		return KErrNone;
		};

	inline TInt SetTriggerKeyScanCode(TUint aTriggerKeyScanCode)
		{
		iTriggerKeyScanCode = aTriggerKeyScanCode;
		return KErrNone;
		};

	inline TInt SetTriggerSystemEvent(const TUid& aTriggerSystemEvent)
		{
		iTriggerSystemEvent = aTriggerSystemEvent;
		return KErrNone;
		};

	inline TInt SetDelayTime(TInt aDelayTime)
		{
		if (aDelayTime < 0)
			{
			return KErrArgument;
			}
		iDelayTime = aDelayTime;
		return KErrNone;
		};

	inline TInt SetNumberOfShots(TInt aNumberOfShots)
		{
		if (aNumberOfShots < 1)
			{
			return KErrArgument;
			}
		iNumberOfShots = aNumberOfShots;
		return KErrNone;
		};

	inline TInt SetIntervalTime(TInt aIntervalTime)
		{
		if (aIntervalTime < 1000)
			{
			return KErrArgument;
			}
		iIntervalTime = aIntervalTime;
		return KErrNone;
		};

	inline TInt SetEnableConfirmationNote(TBool aEnableConfirmationNote)
		{
		iEnableConfirmationNote = aEnableConfirmationNote;
		return KErrNone;
		};

	inline TInt SetEnableSuperimposedText(TBool aEnableSuperimposedText)
		{
		iEnableSuperimposedText = aEnableSuperimposedText;
		return KErrNone;
		};

	inline TInt SetEnableVibrationNotification(TBool aEnableVibrationNotification)
		{
		iEnableVibrationNotification = aEnableVibrationNotification;
		return KErrNone;
		};

	inline TInt SetPngBitsPerPixel(TInt aPngBitsPerPixel)
		{
		if (aPngBitsPerPixel != 1 && aPngBitsPerPixel != 4 && aPngBitsPerPixel != 8 && aPngBitsPerPixel != 24)
			{
			return KErrArgument;
			}
		iPngBitsPerPixel = aPngBitsPerPixel;
		return KErrNone;
		};

	inline TInt SetPngCompressLevel(TPngEncodeData::TPngCompressLevel aPngCompressLevel)
		{
		iPngCompressLevel = aPngCompressLevel;
		return KErrNone;
		};

	inline TInt SetBmpBitsPerPixel(TInt aBmpBitsPerPixel)
		{
		if (aBmpBitsPerPixel != 1 && aBmpBitsPerPixel != 4 && aBmpBitsPerPixel != 8 && aBmpBitsPerPixel != 24)
			{
			return KErrArgument;
			}
		iBmpBitsPerPixel = aBmpBitsPerPixel;
		return KErrNone;
		};

	inline TInt SetMbmDisplayMode(TDisplayMode aMbmDisplayMode)
		{
		if (aMbmDisplayMode == ERgb)
			{
			return KErrArgument;
			}
		iMbmDisplayMode = aMbmDisplayMode;
		return KErrNone;
		};

	inline TInt SetJpegQualityFactor(TInt aJpegQualityFactor)
		{
		if (aJpegQualityFactor < 0 || 100 < aJpegQualityFactor)
			{
			return KErrArgument;
			}
		iJpegQualityFactor = aJpegQualityFactor;
		return KErrNone;
		};

	inline TInt SetJpegSampleScheme(TJpegImageData::TColorSampling aJpegSampleScheme)
		{
		iJpegSampleScheme = aJpegSampleScheme;
		return KErrNone;
		};

	inline TInt SetJpegComment(const TDesC& aJpegComment)
		{
		HBufC* tmpJpegComment = aJpegComment.Alloc();
		if (tmpJpegComment == NULL)
			{
			return KErrNoMemory;
			}
		delete iJpegComment;
		iJpegComment = tmpJpegComment;
		return KErrNone;
		};

	inline const TUid& ImageFormat(void) const		{return iImageFormat;};
	inline TBool EnableThreading(void) const		{return iEnableThreading;};
	inline TBool EnableSaveToExternal(void) const	{return iEnableSaveToExternal;};
	inline const TDesC& SavedDirectoryName(void) const	{return *iSavedDirectoryName;};
	inline const TDesC& SavedFileName(void) const	{return *iSavedFileName;};
	inline TUint TriggerKeyScanCode(void) const		{return iTriggerKeyScanCode;};
	inline const TUid& TriggerSystemEvent(void) const	{return iTriggerSystemEvent;};
	inline TInt DelayTime(void) const				{return iDelayTime;};
	inline TInt NumberOfShots(void) const			{return iNumberOfShots;};
	inline TInt IntervalTime(void) const			{return iIntervalTime;};
	inline TBool EnableConfirmationNote(void) const	{return iEnableConfirmationNote;};
	inline TBool EnableSuperimposedText(void) const	{return iEnableSuperimposedText;};
	inline TBool EnableVibrationNotification(void) const	{return iEnableVibrationNotification;};

	inline TInt PngBitsPerPixel(void) const			{return iPngBitsPerPixel;};
	inline TPngEncodeData::TPngCompressLevel PngCompressLevel(void) const	{return iPngCompressLevel;};
	inline TInt BmpBitsPerPixel(void) const			{return iBmpBitsPerPixel;};
	inline TDisplayMode MbmDisplayMode(void) const	{return iMbmDisplayMode;};
	inline TInt JpegQualityFactor(void) const		{return iJpegQualityFactor;};
	inline TJpegImageData::TColorSampling JpegSampleScheme(void) const	{return iJpegSampleScheme;};
	inline const TDesC& JpegComment(void) const		{return *iJpegComment;};

private:
	// Settings (Encoding)
	TUid iImageFormat;
	TBool iEnableThreading;

	// Settings (File)
	TBool iEnableSaveToExternal;
	HBufC* iSavedDirectoryName;
	HBufC* iSavedFileName;

	// Settings (Trigger)
	TUint iTriggerKeyScanCode;
	TUid iTriggerSystemEvent;
	TInt iDelayTime;

	// Settings (Repeat)
	TInt iNumberOfShots;
	TInt iIntervalTime;

	// Settings (Notification)
	TBool iEnableConfirmationNote;
	TBool iEnableSuperimposedText;
	TBool iEnableVibrationNotification;

	// Image Settings (PNG)
	TInt iPngBitsPerPixel;
	TPngEncodeData::TPngCompressLevel iPngCompressLevel;

	// Image Settings (BMP)
	TInt iBmpBitsPerPixel;

	// Image Settings (MBM)
	TDisplayMode iMbmDisplayMode;

	// Image Settings (JPEG)
	TInt iJpegQualityFactor;
	TJpegImageData::TColorSampling iJpegSampleScheme;
	HBufC* iJpegComment;
	};

#endif	// GETPIXELSSETTINGS_H
