/**
 *	@file	GetPixelsSettingsContainer.h
 *	@author	ENDO
 */

#ifndef GETPIXELSSETTINGSCONTAINER_H
#define GETPIXELSSETTINGSCONTAINER_H

#include <aknsettingitemlist.h>	// for CAknSettingItemList.
#include <akntabobserver.h>		// for MAknTabObserver.
#include <CAknMemorySelectionSettingItem.h>	// for CAknMemorySelectionDialog::TMemory.

#include "GetPixels.hrh"		// for ESettingsTabNum.

class TGetPixelsSettings
	{
public:
	inline TGetPixelsSettings(void) :
		iImageFormat(ESettingsImageFormatPng),
		iForceThreading(ETrue),
		iSaveLocation(CAknMemorySelectionDialog::EPhoneMemory),
		iTriggerKey(ESettingsTriggerKeyEditPlusOK),
		iTriggerEvent(ESettingsTriggerEventNone),
		iDelayTime(0),
		iNumberOfShots(1),
		iIntervalTime(1),
		iConfirmationNote(EFalse),
		iSuperimposedText(ETrue),
		iVibrationNotification(EFalse)
		{
		};

public:
	TInt iImageFormat;
	TBool iForceThreading;

	CAknMemorySelectionDialog::TMemory iSaveLocation;
	TBuf<KMaxSavedDirectoryNameLength> iSavedDirectoryName;
	TBuf<KMaxSavedFileNameLength> iSavedFileName;

	TInt iTriggerKey;
	TInt iTriggerEvent;
	TInt iDelayTime;

	TInt iNumberOfShots;
	TInt iIntervalTime;

	TBool iConfirmationNote;
	TBool iSuperimposedText;
	TBool iVibrationNotification;
	};

class MGetPixelsSettingItemListObserver
	{
public:
	virtual void NotifyEditItemL(TInt aIndex, TBool aCalledFromMenu) = 0;
	};

class CAknSettingItem;
class CGetPixelsSettingItemList : public CAknSettingItemList
	{
public:
	CGetPixelsSettingItemList(MGetPixelsSettingItemListObserver& aObserver, TGetPixelsSettings& aSettings);

public:
	// from CAknSettingItemList.
	CAknSettingItem* CreateSettingItemL(TInt aIdentifier);
	void EditItemL(TInt aIndex, TBool aCalledFromMenu);

private:
	// from CAknSettingItemList.
	void SizeChanged(void);

private:
	MGetPixelsSettingItemListObserver& iObserver;
	TGetPixelsSettings& iSettings;
	};

class CAknNavigationDecorator;
class CAknTabGroup;
class CGetPixelsSettings;
class CGetPixelsAppUi;
class CGetPixelsSettingsContainer : public CCoeControl, public MAknTabObserver, public MGetPixelsSettingItemListObserver
	{
public:
	static CGetPixelsSettingsContainer* NewL(const TRect& aRect);
	virtual ~CGetPixelsSettingsContainer();

private:
	CGetPixelsSettingsContainer(void);
	void ConstructL(const TRect& aRect);

public:
	void SetActiveTab(TInt aIndex);
	void ChangeSelectedItemL(void);
	void ReadSettingsFromModelL(const CGetPixelsSettings& aModel);
	void WriteSettingsToModelL(CGetPixelsSettings& aModel) const;

private:
	// from CCoeControl.
	TInt CountComponentControls(void) const;
	CCoeControl* ComponentControl(TInt aIndex) const;
	TKeyResponse OfferKeyEventL(const TKeyEvent& aKeyEvent, TEventCode aType);

private:
	// from MAknTabObserver.
	void TabChangedL(TInt aIndex);

public:
	// from MGetPixelsSettingItemListObserver.
	void NotifyEditItemL(TInt aIndex, TBool aCalledFromMenu);

private:
	TFixedArray<CGetPixelsSettingItemList*, ESettingsTabNum> iSettingItemLists;
	TInt iCurrentTabIndex;
	CAknNavigationDecorator* iNavigationDecorator;
	CAknTabGroup* iTabGroup;
	TGetPixelsSettings iSettings;
	CGetPixelsAppUi& iAppUi;
	};

#endif // GETPIXELSSETTINGSCONTAINER_H
