/**
 *	@file	GetPixelsDocument.cpp
 *	@author	ENDO
 */

#include "GetPixelsAppUi.h"
#include "GetPixelsDocument.h"
#include "GetPixelsSettings.h"
#include "MyAssert.h"

CGetPixelsDocument* CGetPixelsDocument::NewL(CEikApplication& aApp)
	{
	CGetPixelsDocument* self = NewLC(aApp);
	CleanupStack::Pop(self);
	return self;
	}

CGetPixelsDocument* CGetPixelsDocument::NewLC(CEikApplication& aApp)
	{
	CGetPixelsDocument* self = new (ELeave) CGetPixelsDocument(aApp);

	CleanupStack::PushL(self);
	self->ConstructL();
	return self;
	}

CGetPixelsDocument::CGetPixelsDocument(CEikApplication& aApp) :
	CAknDocument(aApp)
	{
	}

void CGetPixelsDocument::ConstructL(void)
	{
	iModel = CGetPixelsSettings::NewL();
	}

CGetPixelsDocument::~CGetPixelsDocument()
	{
	delete iModel;
	}

CEikAppUi* CGetPixelsDocument::CreateAppUiL(void)
	{
	return (static_cast<CEikAppUi*>(new(ELeave) CGetPixelsAppUi));
	}

CFileStore* CGetPixelsDocument::OpenFileL(TBool aDoOpen,const TDesC& aFilename,RFs& aFs)
	{
	return CEikDocument::OpenFileL(aDoOpen, aFilename, aFs);
	}

void CGetPixelsDocument::StoreL(CStreamStore& aStore, CStreamDictionary& aStreamDic) const
	{
	RStoreWriteStream stream;
	TStreamId id = stream.CreateLC(aStore);

	MY_ASSERT(iModel != NULL);
	iModel->ExternalizeL(stream);

	aStreamDic.AssignL(Application()->AppDllUid(), id);

	CleanupStack::PopAndDestroy();	// RStoreWriteStream stream
	}

void CGetPixelsDocument::RestoreL(const CStreamStore& aStore, const CStreamDictionary& aStreamDic)
	{
	TStreamId id = aStreamDic.At(Application()->AppDllUid());
	RStoreReadStream stream;
	stream.OpenLC(aStore, id);

	CGetPixelsSettings* newModel = CGetPixelsSettings::NewLC();
	TRAPD(ret, newModel->InternalizeL(stream));
	if (ret == KErrNone)
		{
		delete iModel;
		iModel = newModel;
		CleanupStack::Pop(newModel);
		}
	else
		{
		CleanupStack::PopAndDestroy(newModel);
		}

	CleanupStack::PopAndDestroy();	// RStoreReadStream stream
	}
