/**
 *	@file	GetPixelsImageSettingsContainer.cpp
 *	@author	ENDO
 */

#include <e32base.h>
#include <barsread.h>	// for TResourceReader.
#include <eikspane.h>	// for CEikStatusPane.

#include <aknnavi.h>			// for CAknNavigationControlContainer.
#include <aknnavide.h>			// for CAknNavigationDecorator.
#include <akntabgrp.h>			// for CAknTabGroup.
#include <aknsettingitemlist.h>	// for CAknSettingItemList.
#include <akntextsettingpage.h>	// for CAknTextSettingPage.

#include <GetPixels.rsg>
#include "GetPixels.hrh"
#include "GetPixelsImageSettingsContainer.h"
#include "GetPixelsSettings.h"
#include "GetPixelsAppUi.h"
#include "MyAssert.h"

CGetPixelsImageSettingsContainer* CGetPixelsImageSettingsContainer::NewL(const TRect& aRect)
	{
	CGetPixelsImageSettingsContainer* self = new(ELeave) CGetPixelsImageSettingsContainer;
	CleanupStack::PushL(self);
	self->ConstructL(aRect);
	CleanupStack::Pop(self);
	return self;
	}

void CGetPixelsImageSettingsContainer::ConstructL(const TRect& aRect)
	{
	CreateWindowL();

	for (TInt i = 0; i < iSettingItemLists.Count(); i++)
		{
		LOCAL_D const TInt resourceIds[] =
			{
			R_GETPIXELS_SETTINGITEMLIST_IMAGESETTINGS_PNG,
			R_GETPIXELS_SETTINGITEMLIST_IMAGESETTINGS_BMP,
			R_GETPIXELS_SETTINGITEMLIST_IMAGESETTINGS_MBM,
			R_GETPIXELS_SETTINGITEMLIST_IMAGESETTINGS_JPEG,
			};
		MY_ASSERT(sizeof(resourceIds) / sizeof(resourceIds[0]) == iSettingItemLists.Count());

		iSettingItemLists[i] = new(ELeave) CGetPixelsImageSettingItemList(*this, iSettings);
		iSettingItemLists[i]->SetMopParent(this);
		iSettingItemLists[i]->ConstructFromResourceL(resourceIds[i]);
		iSettingItemLists[i]->ListBox()->CreateScrollBarFrameL(ETrue);
		iSettingItemLists[i]->ListBox()->ScrollBarFrame()->SetScrollBarVisibilityL(CEikScrollBarFrame::EOff, CEikScrollBarFrame::EAuto);
		iSettingItemLists[i]->MakeVisible(EFalse);
		}

	iCurrentTabIndex = 0;
	iSettingItemLists[iCurrentTabIndex]->MakeVisible(ETrue);

		{
		TUid naviPaneUid;
		naviPaneUid.iUid = EEikStatusPaneUidNavi;

		CEikStatusPane* statusPane = (static_cast<CGetPixelsAppUi*>(iCoeEnv->AppUi()))->StatusPane();
		MY_ASSERT(statusPane != NULL);
		CEikStatusPaneBase::TPaneCapabilities subPane = statusPane->PaneCapabilities(naviPaneUid);

		if (subPane.IsPresent() && subPane.IsAppOwned())
			{
			CAknNavigationControlContainer* naviPane = static_cast<CAknNavigationControlContainer*>(statusPane->ControlL(naviPaneUid));

				{
				TResourceReader reader;
				iCoeEnv->CreateResourceReaderLC(reader, R_GETPIXELS_TABGROUP_IMAGESETTINGS);

				iNavigationDecorator = naviPane->CreateTabGroupL(reader);

				CleanupStack::PopAndDestroy();
				}

			naviPane->PushL(*iNavigationDecorator);

			iTabGroup = reinterpret_cast<CAknTabGroup*>(iNavigationDecorator->DecoratedControl());
			MY_ASSERT(iTabGroup != NULL);
			iTabGroup->SetObserver(this);
			}
		}

	SetRect(aRect);
	ActivateL();
	}

CGetPixelsImageSettingsContainer::CGetPixelsImageSettingsContainer(void) :
	iAppUi(*reinterpret_cast<CGetPixelsAppUi*>(CCoeEnv::Static()->AppUi()))
	{
	}

CGetPixelsImageSettingsContainer::~CGetPixelsImageSettingsContainer()
	{
	for (TInt i = 0; i < iSettingItemLists.Count(); i++)
		{
		delete iSettingItemLists[i];
		}

	if (iNavigationDecorator != NULL)
		{
		iTabGroup->SetObserver(NULL);
		delete iNavigationDecorator;
		}
	}

TInt CGetPixelsImageSettingsContainer::CountComponentControls(void) const
	{
	return iSettingItemLists.Count();
	}

CCoeControl* CGetPixelsImageSettingsContainer::ComponentControl(TInt aIndex) const
	{
	if (0 <= aIndex && aIndex < iSettingItemLists.Count())
		{
		return iSettingItemLists[aIndex];
		}

	return NULL;
	}

TKeyResponse CGetPixelsImageSettingsContainer::OfferKeyEventL(const TKeyEvent& aKeyEvent, TEventCode aType)
	{
	MY_ASSERT(iTabGroup != NULL);
	if (iTabGroup->OfferKeyEventL(aKeyEvent, aType) == EKeyWasConsumed)
		{
		return EKeyWasConsumed;
		}

	MY_ASSERT(iSettingItemLists[iCurrentTabIndex] != NULL);
	if (iSettingItemLists[iCurrentTabIndex]->OfferKeyEventL(aKeyEvent, aType) == EKeyWasConsumed)
		{
		return EKeyWasConsumed;
		}

	return EKeyWasNotConsumed;
	}

void CGetPixelsImageSettingsContainer::TabChangedL(TInt aIndex)
	{
	iSettingItemLists[iCurrentTabIndex]->MakeVisible(EFalse);

	iCurrentTabIndex = aIndex;

	iSettingItemLists[iCurrentTabIndex]->MakeVisible(ETrue);
	}

void CGetPixelsImageSettingsContainer::SetActiveTab(TInt aIndex)
	{
	if (iTabGroup != NULL)
		{
		iTabGroup->SetActiveTabByIndex(aIndex);
		TRAPD(ret, TabChangedL(aIndex));
		}
	}

void CGetPixelsImageSettingsContainer::ChangeSelectedItemL(void)
	{
	MY_ASSERT(iSettingItemLists[iCurrentTabIndex] != NULL);
	iSettingItemLists[iCurrentTabIndex]->EditItemL(iSettingItemLists[iCurrentTabIndex]->ListBox()->CurrentItemIndex(), ETrue);
	}

void CGetPixelsImageSettingsContainer::ReadSettingsFromModelL(const CGetPixelsSettings& aModel)
	{
	// PNG Colors
	switch (aModel.PngBitsPerPixel())
		{
	case 8:
		iSettings.iPngColors = EImageSettingsPngColors256;
		break;
	case 24:
		iSettings.iPngColors = EImageSettingsPngColors16M;
		break;
	default:
		break;
		}

	// PNG Compress level
	switch (aModel.PngCompressLevel())
		{
	case TPngEncodeData::EDefaultCompression:
		iSettings.iPngCompressLevel = EImageSettingsPngCompressLevelDefaultCompression;
		break;
	case TPngEncodeData::EBestCompression:
		iSettings.iPngCompressLevel = EImageSettingsPngCompressLevelBestCompression;
		break;
	case TPngEncodeData::EBestSpeed:
		iSettings.iPngCompressLevel = EImageSettingsPngCompressLevelBestSpeed;
		break;
	case TPngEncodeData::ENoCompression:
		iSettings.iPngCompressLevel = EImageSettingsPngCompressLevelNoCompression;
		break;
	default:
		break;
		}

	// BMP Colors
	switch (aModel.BmpBitsPerPixel())
		{
	case 8:
		iSettings.iBmpColors = EImageSettingsBmpColors256;
		break;
	case 24:
		iSettings.iBmpColors = EImageSettingsBmpColors16M;
		break;
	default:
		break;
		}

	// MBM Colors
	switch (aModel.MbmDisplayMode())
		{
	case ENone:
		iSettings.iMbmColors = EImageSettingsMbmColorsDefault;
		break;
	case EColor256:
		iSettings.iMbmColors = EImageSettingsMbmColors256;
		break;
	case EColor4K:
		iSettings.iMbmColors = EImageSettingsMbmColors4K;
		break;
	case EColor64K:
		iSettings.iMbmColors = EImageSettingsMbmColors64K;
		break;
	case EColor16M:
		iSettings.iMbmColors = EImageSettingsMbmColors16M;
		break;
	default:
		break;
		}

	// JPEG Quality
	iSettings.iJpegQuality = aModel.JpegQualityFactor();

	// JPEG Color format
	switch (aModel.JpegSampleScheme())
		{
	case TJpegImageData::EColor420:
		iSettings.iJpegColorSampling = EImageSettingsJpegColorSampling420;
		break;
	case TJpegImageData::EColor422:
		iSettings.iJpegColorSampling = EImageSettingsJpegColorSampling422;
		break;
	case TJpegImageData::EColor444:
		iSettings.iJpegColorSampling = EImageSettingsJpegColorSampling444;
		break;
	default:
		break;
		}

	// JPEG Comment
	iSettings.iJpegComment = aModel.JpegComment();

	for (TInt i = 0; i < iSettingItemLists.Count(); i++)
		{
		iSettingItemLists[i]->LoadSettingsL();
		}
	}

void CGetPixelsImageSettingsContainer::WriteSettingsToModelL(CGetPixelsSettings& aModel) const
	{
	TInt ret;

	for (TInt i = 0; i < iSettingItemLists.Count(); i++)
		{
		iSettingItemLists[i]->StoreSettingsL();
		}

	// PNG Colors
	LOCAL_D const TInt tablePngBitsPerPixel[] = {8, 24};
	MY_ASSERT(iSettings.iPngColors < sizeof(tablePngBitsPerPixel) / sizeof(tablePngBitsPerPixel[0]));
	ret = aModel.SetPngBitsPerPixel(tablePngBitsPerPixel[iSettings.iPngColors]);
	MY_ASSERT(ret == KErrNone);

	// PNG Compress level
	LOCAL_D const TPngEncodeData::TPngCompressLevel tablePngCompressLevel[] =
		{
		TPngEncodeData::EDefaultCompression,
		TPngEncodeData::EBestCompression,
		TPngEncodeData::EBestSpeed,
		TPngEncodeData::ENoCompression,
		};
	MY_ASSERT(iSettings.iPngCompressLevel < sizeof(tablePngCompressLevel) / sizeof(tablePngCompressLevel[0]));
	ret = aModel.SetPngCompressLevel(tablePngCompressLevel[iSettings.iPngCompressLevel]);
	MY_ASSERT(ret == KErrNone);

	// BMP Colors
	LOCAL_D const TInt tableBmpBitsPerPixel[] = {8, 24};
	MY_ASSERT(iSettings.iBmpColors < sizeof(tableBmpBitsPerPixel) / sizeof(tableBmpBitsPerPixel[0]));
	ret = aModel.SetBmpBitsPerPixel(tableBmpBitsPerPixel[iSettings.iBmpColors]);
	MY_ASSERT(ret == KErrNone);

	// MBM Colors
	LOCAL_D const TDisplayMode tableMbmDisplayMode[] =
		{
		ENone,
		EColor256,
		EColor4K,
		EColor64K,
		EColor16M,
		};
	MY_ASSERT(iSettings.iMbmColors < sizeof(tableMbmDisplayMode) / sizeof(tableMbmDisplayMode[0]));
	ret = aModel.SetMbmDisplayMode(tableMbmDisplayMode[iSettings.iMbmColors]);
	MY_ASSERT(ret == KErrNone);

	// JPEG Quality
	ret = aModel.SetJpegQualityFactor(iSettings.iJpegQuality);
	MY_ASSERT(ret == KErrNone);

	// JPEG Color format
	LOCAL_D const TJpegImageData::TColorSampling tableJpegSampleScheme[] =
		{
		TJpegImageData::EColor420,
		TJpegImageData::EColor422,
		TJpegImageData::EColor444,
		};
	MY_ASSERT(iSettings.iJpegColorSampling < sizeof(tableJpegSampleScheme) / sizeof(tableJpegSampleScheme[0]));
	ret = aModel.SetJpegSampleScheme(tableJpegSampleScheme[iSettings.iJpegColorSampling]);
	MY_ASSERT(ret == KErrNone);

	// JPEG Comment
	ret = aModel.SetJpegComment(iSettings.iJpegComment);
	MY_ASSERT(ret == KErrNone);
	}

void CGetPixelsImageSettingsContainer::NotifyEditItemL(TInt /* aIndex */, TBool /* aCalledFromMenu */)
	{
	WriteSettingsToModelL(*iAppUi.GetModel());
	}

CGetPixelsImageSettingItemList::CGetPixelsImageSettingItemList(MGetPixelsImageSettingItemListObserver& aObserver, TGetPixelsImageSettings& aSettings) :
	iObserver(aObserver),
	iSettings(aSettings)
	{
	}

void CGetPixelsImageSettingItemList::SizeChanged(void)
	{
	if (ListBox() != NULL)
		{
		ListBox()->SetRect(Rect());
		}
	}

CAknSettingItem* CGetPixelsImageSettingItemList::CreateSettingItemL(TInt aIdentifier)
	{
	CAknSettingItem* settingItem = NULL;

	switch (aIdentifier)
		{
	case EImageSettingsPngColors:
		settingItem = new(ELeave) CAknEnumeratedTextPopupSettingItem(aIdentifier, iSettings.iPngColors);
		break;

	case EImageSettingsPngCompressLevel:
		settingItem = new(ELeave) CAknEnumeratedTextPopupSettingItem(aIdentifier, iSettings.iPngCompressLevel);
		break;

	case EImageSettingsBmpColors:
		settingItem = new(ELeave) CAknEnumeratedTextPopupSettingItem(aIdentifier, iSettings.iBmpColors);
		break;

	case EImageSettingsMbmColors:
		settingItem = new(ELeave) CAknEnumeratedTextPopupSettingItem(aIdentifier, iSettings.iMbmColors);
		break;

	case EImageSettingsJpegQuality:
		settingItem = new(ELeave) CAknSliderSettingItem(aIdentifier, iSettings.iJpegQuality);
		break;

	case EImageSettingsJpegColorSampling:
		settingItem = new(ELeave) CAknEnumeratedTextPopupSettingItem(aIdentifier, iSettings.iJpegColorSampling);
		break;

	case EImageSettingsJpegComment:
		settingItem = new(ELeave) CAknTextSettingItem(aIdentifier, iSettings.iJpegComment);
		settingItem->SetSettingPageFlags(settingItem->SettingPageFlags() | CAknTextSettingPage::EZeroLengthAllowed);
		break;

	default:
		MY_ASSERT(EFalse);
		break;
		}

	return settingItem;
	}

void CGetPixelsImageSettingItemList::EditItemL(TInt aIndex, TBool aCalledFromMenu)
	{
	CAknSettingItemList::EditItemL(aIndex, aCalledFromMenu);
	(*SettingItemArray())[aIndex]->StoreL();

	iObserver.NotifyEditItemL(aIndex, aCalledFromMenu);
	}
