/**
 *	@file	GetPixelsImageSettingsView.cpp
 *	@author	ENDO
 */

#include <aknview.h>		// for CAknView.
#include <aknviewappui.h>	// for CAknViewAppUi.

#include <GetPixels.rsg>
#include "GetPixels.hrh"
#include "GetPixelsDocument.h"
#include "GetPixelsAppUi.h"
#include "GetPixelsImageSettingsView.h"
#include "GetPixelsImageSettingsContainer.h"
#include "GetPixelsSettings.h"

CGetPixelsImageSettingsView* CGetPixelsImageSettingsView::NewL(void)
	{
	CGetPixelsImageSettingsView* self = new(ELeave) CGetPixelsImageSettingsView;
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop(self);
	return self;
	}

CGetPixelsImageSettingsView::CGetPixelsImageSettingsView(void) :
	iAppUi(*reinterpret_cast<CGetPixelsAppUi*>(AppUi()))
	{
	}

void CGetPixelsImageSettingsView::ConstructL(void)
	{
	BaseConstructL(R_GETPIXELS_VIEW_SETTINGS);
	}

CGetPixelsImageSettingsView::~CGetPixelsImageSettingsView()
	{
	delete iContainer;
	}

TUid CGetPixelsImageSettingsView::Id(void) const
	{
	return TUid::Uid(EGetPixelsImageSettingsViewId);
	}

void CGetPixelsImageSettingsView::DoActivateL(const TVwsViewId& /* aPrevViewId */, TUid /* aCustomMessageId */, const TDesC8& /* aCustomMessage */)
	{
	if (iContainer == NULL)
		{
		iContainer = CGetPixelsImageSettingsContainer::NewL(iAppUi.ClientRect());
		iContainer->SetMopParent(this);
		iAppUi.AddToStackL(*this, iContainer);
		}

	iContainer->ReadSettingsFromModelL(*iAppUi.GetModel());

	if (iAppUi.GetModel()->ImageFormat() == KImageTypePNGUid)
		{
		iContainer->SetActiveTab(ESettingsImageFormatPng);
		}
	else if (iAppUi.GetModel()->ImageFormat() == KImageTypeBMPUid)
		{
		iContainer->SetActiveTab(ESettingsImageFormatBmp);
		}
	else if (iAppUi.GetModel()->ImageFormat() == KImageTypeMBMUid)
		{
		iContainer->SetActiveTab(ESettingsImageFormatMbm);
		}
	else if (iAppUi.GetModel()->ImageFormat() == KImageTypeJPGUid)
		{
		iContainer->SetActiveTab(ESettingsImageFormatJpeg);
		}
	}

void CGetPixelsImageSettingsView::DoDeactivate(void)
	{
	if (iContainer != NULL)
		{
//		iContainer->WriteSettingsToModelL(*iAppUi.GetModel());

		iAppUi.RemoveFromStack(iContainer);
		delete iContainer;
		iContainer = NULL;
		}
	}

void CGetPixelsImageSettingsView::HandleCommandL(TInt aCommand)
	{
	switch (aCommand)
		{
	case ECommandChange:
		iContainer->ChangeSelectedItemL();
		break;

	case EAknSoftkeyBack:
		iAppUi.HandleCommandL(ECommandBackToPreviousView);
		break;

	default:
		iAppUi.HandleCommandL(aCommand);
		break;
		}
	}
