/**
 *	@file	GetPixelsMainView.cpp
 *	@author	ENDO
 */

#include <aknview.h>		// for CAknView.
#include <aknviewappui.h>	// for CAknViewAppUi.
#include <sendui.h>			// for CSendAppUi.

#include <SendNorm.rsg>		// for R_SENDUI_MENU.

#include <GetPixels.rsg>
#include "GetPixels.hrh"
#include "GetPixelsAppUi.h"
#include "GetPixelsMainView.h"
#include "GetPixelsMainContainer.h"

CGetPixelsMainView* CGetPixelsMainView::NewL(void)
	{
	CGetPixelsMainView* self = new(ELeave) CGetPixelsMainView;
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop(self);
	return self;
	}

CGetPixelsMainView::CGetPixelsMainView(void) :
	iAppUi(*reinterpret_cast<CGetPixelsAppUi*>(AppUi()))
	{
	iSendAppUiCapabilities.iFlags = TSendingCapabilities::ESupportsAttachments;
	}

void CGetPixelsMainView::ConstructL(void)
	{
	BaseConstructL(R_GETPIXELS_VIEW_MAIN);

	iSendAppUi = CSendAppUi::NewL(ECommandSendAppUiBase);
	}

CGetPixelsMainView::~CGetPixelsMainView()
	{
	delete iContainer;
	delete iSendAppUi;
	}

void CGetPixelsMainView::DrawDeferred(void) const
	{
	if (iContainer != NULL)
		{
		iContainer->DrawDeferred();
		}
	}

TUid CGetPixelsMainView::Id(void) const
	{
	return TUid::Uid(EGetPixelsMainViewId);
	}

void CGetPixelsMainView::DoActivateL(const TVwsViewId& /* aPrevViewId */, TUid /* aCustomMessageId */, const TDesC8& /* aCustomMessage */)
	{
	if (iContainer == NULL)
		{
		iContainer = CGetPixelsMainContainer::NewL(iAppUi.ClientRect(), iAppUi);
		iContainer->SetMopParent(this);
		iAppUi.AddToStackL(*this, iContainer);
		}
	}

void CGetPixelsMainView::DoDeactivate(void)
	{
	if (iContainer != NULL)
		{
		iAppUi.RemoveFromStack(iContainer);
		delete iContainer;
		iContainer = NULL;
		}
	}

void CGetPixelsMainView::HandleCommandL(TInt aCommand)
	{
	switch (aCommand)
		{
	case EAknSoftkeyBack:
		iAppUi.HandleCommandL(EEikCmdExit);
		break;

	default:
		if (!HandleSendAppUiCommandL(aCommand))
			{
			iAppUi.HandleCommandL(aCommand);
			}
		}
	}

void CGetPixelsMainView::DynInitMenuPaneL(TInt aMenuId, CEikMenuPane* aMenuPane)
	{
	switch (aMenuId)
		{
	case R_GETPIXELS_MENUPANE_MAIN:
		if (iAppUi.CapturedImageFilePath().Length() > 0)
			{
			iSendAppUi->DisplaySendMenuItemL(*aMenuPane, 0, iSendAppUiCapabilities);
			}
		break;

	case R_SENDUI_MENU:
		iSendAppUi->DisplaySendCascadeMenuL(*aMenuPane, NULL);
		break;
		}
	}

TBool CGetPixelsMainView::HandleSendAppUiCommandL(TInt aCommand)
	{
	if (!iSendAppUi->CommandIsValidL(aCommand, iSendAppUiCapabilities))
		{
		return EFalse;
		}

	CDesCArrayFlat* attachments = new(ELeave) CDesCArrayFlat(1);
	CleanupStack::PushL(attachments);
    attachments->AppendL(iAppUi.CapturedImageFilePath());

	iSendAppUi->CreateAndSendMessageL(aCommand, NULL, attachments);

	CleanupStack::PopAndDestroy(attachments);

	return ETrue;
	}
