/**
 *	@file	GetPixelsSettings.cpp
 *	@author	ENDO
 */

#include <e32base.h>
#include <f32file.h>
#include <imageconversion.h>

#include "GetPixelsSettings.h"

LOCAL_D const TUint32 KUidValueImageFormat                 = static_cast<TUint32>('I' | ('F' << 8));
LOCAL_D const TUint32 KUidValueEnableThreading             = static_cast<TUint32>('E' | ('T' << 8));
LOCAL_D const TUint32 KUidValueEnableSaveToExternal        = static_cast<TUint32>('E' | ('S' << 8) | ('T' << 16) | ('E' << 24));
LOCAL_D const TUint32 KUidValueSavedDirectoryName          = static_cast<TUint32>('S' | ('D' << 8) | ('N' << 16));
LOCAL_D const TUint32 KUidValueSavedFileName               = static_cast<TUint32>('S' | ('F' << 8) | ('N' << 16));
LOCAL_D const TUint32 KUidValueTriggerKeyScanCode          = static_cast<TUint32>('T' | ('K' << 8) | ('S' << 16) | ('C' << 24));
LOCAL_D const TUint32 KUidValueTriggerSystemEvent          = static_cast<TUint32>('T' | ('S' << 8) | ('E' << 16));
LOCAL_D const TUint32 KUidValueDelayTimeOfFirstShot        = static_cast<TUint32>('D' | ('T' << 8) | ('F' << 16) | ('S' << 24));
LOCAL_D const TUint32 KUidValueNumberOfShots               = static_cast<TUint32>('N' | ('O' << 8) | ('S' << 16));
LOCAL_D const TUint32 KUidValueIntervalTimeBetweenShots    = static_cast<TUint32>('I' | ('T' << 8) | ('B' << 16) | ('S' << 24));
LOCAL_D const TUint32 KUidValueEnableConfirmationNote      = static_cast<TUint32>('E' | ('C' << 8) | ('N' << 16));
LOCAL_D const TUint32 KUidValueEnableSuperimposedText      = static_cast<TUint32>('E' | ('S' << 8) | ('T' << 16));
LOCAL_D const TUint32 KUidValueEnableVibrationNotification = static_cast<TUint32>('E' | ('V' << 8) | ('N' << 16));
LOCAL_D const TUint32 KUidValuePngBitsPerPixel             = static_cast<TUint32>('P' | ('B' << 8) | ('P' << 16) | ('P' << 24));
LOCAL_D const TUint32 KUidValuePngCompressLevel            = static_cast<TUint32>('P' | ('C' << 8) | ('L' << 16));
LOCAL_D const TUint32 KUidValueBmpBitsPerPixel             = static_cast<TUint32>('B' | ('B' << 8) | ('P' << 16) | ('P' << 24));
LOCAL_D const TUint32 KUidValueMbmDisplayMode              = static_cast<TUint32>('M' | ('D' << 8) | ('M' << 16));
LOCAL_D const TUint32 KUidValueJpegQualityFactor           = static_cast<TUint32>('J' | ('Q' << 8) | ('F' << 16));
LOCAL_D const TUint32 KUidValueJpegSampleScheme            = static_cast<TUint32>('J' | ('S' << 8) | ('S' << 16));
LOCAL_D const TUint32 KUidValueJpegComment                 = static_cast<TUint32>('J' | ('C' << 8));
LOCAL_D const TUint32 KUidValueEndOfStream                 = static_cast<TUint32>('E' | ('O' << 8) | ('S' << 16));

_LIT(KDefaultSavedDirectoryName, "GetPixels");
_LIT(KDefaultSavedFileName, "GetPixels_%F%Y%M%D_%H%T%S");

CGetPixelsSettings::CGetPixelsSettings(void) :
	iImageFormat(KImageTypePNGUid),
	iEnableThreading(ETrue),
	iEnableSaveToExternal(EFalse),
	iSavedDirectoryName(NULL),
	iSavedFileName(NULL),
	iTriggerKeyScanCode(EStdKeyDevice3),
	iTriggerSystemEvent(TUid::Null()),
	iDelayTime(0),
	iNumberOfShots(1),
	iIntervalTime(1000),
	iEnableConfirmationNote(EFalse),
	iEnableSuperimposedText(ETrue),
	iEnableVibrationNotification(EFalse),
	iPngBitsPerPixel(24),
	iPngCompressLevel(TPngEncodeData::EDefaultCompression),
	iBmpBitsPerPixel(24),
	iMbmDisplayMode(ENone),
	iJpegQualityFactor(100),
	iJpegSampleScheme(TJpegImageData::EColor444),
	iJpegComment(NULL)
	{
	}

CGetPixelsSettings::~CGetPixelsSettings()
	{
	delete iJpegComment;
	delete iSavedFileName;
	delete iSavedDirectoryName;
	}

CGetPixelsSettings* CGetPixelsSettings::NewL(void)
	{
	CGetPixelsSettings* self = CGetPixelsSettings::NewLC();
	CleanupStack::Pop(self);

	return self;
	}

CGetPixelsSettings* CGetPixelsSettings::NewLC(void)
	{
	CGetPixelsSettings* self = new(ELeave) CGetPixelsSettings;
	CleanupStack::PushL(self);
	self->ConstructL();

	return self;
	}

void CGetPixelsSettings::ConstructL(void)
	{
	iSavedDirectoryName = KDefaultSavedDirectoryName().AllocL();
	iSavedFileName = KDefaultSavedFileName().AllocL();
	iJpegComment = KNullDesC().AllocL();
	}

void CGetPixelsSettings::ExternalizeL(RWriteStream& aStream) const
	{
	aStream.WriteUint32L(KUidValueImageFormat);
	aStream.WriteInt32L(ImageFormat().iUid);

	aStream.WriteUint32L(KUidValueEnableThreading);
	aStream.WriteInt32L(static_cast<TInt32>(EnableThreading()));

	aStream.WriteUint32L(KUidValueEnableSaveToExternal);
	aStream.WriteInt32L(static_cast<TInt32>(EnableSaveToExternal()));

	aStream.WriteUint32L(KUidValueSavedDirectoryName);
	aStream << SavedDirectoryName();

	aStream.WriteUint32L(KUidValueSavedFileName);
	aStream << SavedFileName();

	aStream.WriteUint32L(KUidValueTriggerKeyScanCode);
	aStream.WriteInt32L(static_cast<TInt32>(TriggerKeyScanCode()));

	aStream.WriteUint32L(KUidValueTriggerSystemEvent);
	aStream.WriteInt32L(TriggerSystemEvent().iUid);

	aStream.WriteUint32L(KUidValueDelayTimeOfFirstShot);
	aStream.WriteInt32L(static_cast<TInt32>(DelayTime()));

	aStream.WriteUint32L(KUidValueNumberOfShots);
	aStream.WriteInt32L(static_cast<TInt32>(NumberOfShots()));

	aStream.WriteUint32L(KUidValueIntervalTimeBetweenShots);
	aStream.WriteInt32L(static_cast<TInt32>(IntervalTime()));

	aStream.WriteUint32L(KUidValueEnableConfirmationNote);
	aStream.WriteInt32L(static_cast<TInt32>(EnableConfirmationNote()));

	aStream.WriteUint32L(KUidValueEnableSuperimposedText);
	aStream.WriteInt32L(static_cast<TInt32>(EnableSuperimposedText()));

	aStream.WriteUint32L(KUidValueEnableVibrationNotification);
	aStream.WriteInt32L(static_cast<TInt32>(EnableVibrationNotification()));

	aStream.WriteUint32L(KUidValuePngBitsPerPixel);
	aStream.WriteInt32L(static_cast<TInt32>(PngBitsPerPixel()));

	aStream.WriteUint32L(KUidValuePngCompressLevel);
	aStream.WriteInt32L(static_cast<TInt32>(PngCompressLevel()));

	aStream.WriteUint32L(KUidValueBmpBitsPerPixel);
	aStream.WriteInt32L(static_cast<TInt32>(BmpBitsPerPixel()));

	aStream.WriteUint32L(KUidValueMbmDisplayMode);
	aStream.WriteInt32L(static_cast<TInt32>(MbmDisplayMode()));

	aStream.WriteUint32L(KUidValueJpegQualityFactor);
	aStream.WriteInt32L(static_cast<TInt32>(JpegQualityFactor()));

	aStream.WriteUint32L(KUidValueJpegSampleScheme);
	aStream.WriteInt32L(static_cast<TInt32>(JpegSampleScheme()));

	aStream.WriteUint32L(KUidValueJpegComment);
	aStream << JpegComment();

	aStream.WriteUint32L(KUidValueEndOfStream);
	}

void CGetPixelsSettings::InternalizeL(RReadStream& aStream)
	{
	for (;;)
		{
		TUint32 uidValue = 0;
		TRAPD(ret, uidValue = aStream.ReadUint32L());
		if (uidValue == KUidValueEndOfStream || ret != KErrNone)
			{
			break;
			}

		switch (uidValue)
			{
		case KUidValueImageFormat:
				{
				TUid uid;
				uid.iUid = aStream.ReadUint32L();
				User::LeaveIfError(SetImageFormat(uid));
				}
			break;

		case KUidValueEnableThreading:
			User::LeaveIfError(SetEnableThreading(static_cast<TBool>(aStream.ReadInt32L())));
			break;

		case KUidValueEnableSaveToExternal:
			User::LeaveIfError(SetEnableSaveToExternal(static_cast<TBool>(aStream.ReadInt32L())));
			break;

		case KUidValueSavedDirectoryName:
				{
				delete iSavedDirectoryName;
				iSavedDirectoryName = NULL;
				TRAPD(ret, iSavedDirectoryName = HBufC::NewL(aStream, KMaxTInt));
				if (ret != KErrNone)
					{
					iSavedDirectoryName = KNullDesC().AllocL();
					}
				}
			break;

		case KUidValueSavedFileName:
				{
				delete iSavedFileName;
				iSavedFileName = NULL;
				TRAPD(ret, iSavedFileName = HBufC::NewL(aStream, KMaxTInt));
				if (ret != KErrNone)
					{
					iSavedFileName = KNullDesC().AllocL();
					}
				}
			break;

		case KUidValueTriggerKeyScanCode:
			User::LeaveIfError(SetTriggerKeyScanCode(static_cast<TInt>(aStream.ReadInt32L())));
			break;

		case KUidValueTriggerSystemEvent:
				{
				TUid uid;
				uid.iUid = aStream.ReadUint32L();
				User::LeaveIfError(SetTriggerSystemEvent(uid));
				}
			break;

		case KUidValueDelayTimeOfFirstShot:
			User::LeaveIfError(SetDelayTime(static_cast<TInt>(aStream.ReadInt32L())));
			break;

		case KUidValueNumberOfShots:
			User::LeaveIfError(SetNumberOfShots(static_cast<TInt>(aStream.ReadInt32L())));
			break;

		case KUidValueIntervalTimeBetweenShots:
			User::LeaveIfError(SetIntervalTime(static_cast<TInt>(aStream.ReadInt32L())));
			break;

		case KUidValueEnableConfirmationNote:
			User::LeaveIfError(SetEnableConfirmationNote(static_cast<TBool>(aStream.ReadInt32L())));
			break;

		case KUidValueEnableSuperimposedText:
			User::LeaveIfError(SetEnableSuperimposedText(static_cast<TBool>(aStream.ReadInt32L())));
			break;

		case KUidValueEnableVibrationNotification:
			User::LeaveIfError(SetEnableVibrationNotification(static_cast<TBool>(aStream.ReadInt32L())));
			break;

		case KUidValuePngBitsPerPixel:
			User::LeaveIfError(SetPngBitsPerPixel(static_cast<TInt>(aStream.ReadInt32L())));
			break;

		case KUidValuePngCompressLevel:
			User::LeaveIfError(SetPngCompressLevel(static_cast<TPngEncodeData::TPngCompressLevel>(aStream.ReadInt32L())));
			break;

		case KUidValueBmpBitsPerPixel:
			User::LeaveIfError(SetBmpBitsPerPixel(static_cast<TInt>(aStream.ReadInt32L())));
			break;

		case KUidValueMbmDisplayMode:
			User::LeaveIfError(SetMbmDisplayMode(static_cast<TDisplayMode>(aStream.ReadInt32L())));
			break;

		case KUidValueJpegQualityFactor:
			User::LeaveIfError(SetJpegQualityFactor(static_cast<TInt>(aStream.ReadInt32L())));
			break;

		case KUidValueJpegSampleScheme:
			User::LeaveIfError(SetJpegSampleScheme(static_cast<TJpegImageData::TColorSampling>(aStream.ReadInt32L())));
			break;

		case KUidValueJpegComment:
				{
				delete iJpegComment;
				iJpegComment = NULL;
				TRAPD(ret, iJpegComment = HBufC::NewL(aStream, KMaxTInt));
				if (ret != KErrNone)
					{
					iJpegComment = KNullDesC().AllocL();
					}
				}
			break;

		default:
//			User::Leave(KErrGeneral);
			break;
			}
		}
	}
