/**
 *	@file	GetPixelsSettingsContainer.cpp
 *	@author	ENDO
 */

#include <e32base.h>
#include <barsread.h>	// for TResourceReader.
#include <eikspane.h>	// for CEikStatusPane.
#include <sacls.h>		// for KUidChargerStatus.

#include <stringloader.h>
#include <aknnavi.h>			// for CAknNavigationControlContainer.
#include <aknnavide.h>			// for CAknNavigationDecorator.
#include <akntabgrp.h>			// for CAknTabGroup.
#include <aknsettingitemlist.h>	// for CAknSettingItemList.
#include <akntextsettingpage.h>	// for CAknTextSettingPage.
#include <aknnotewrappers.h>	// for CAknInformationNote.
#include <caknmemoryselectionsettingitem.h>	// for CAknMemorySelectionDialog::TMemory.

#include "AknSliderSettingItemTextLabel.h"

#include <GetPixels.rsg>
#include "GetPixels.hrh"
#include "GetPixelsSettingsContainer.h"
#include "GetPixelsSettings.h"
#include "GetPixelsAppUi.h"
#include "GetPixelsUtilities.h"
#include "MyAssert.h"

LOCAL_D const TUint KTableKeyScanCodes[] =
	{
	EStdKeyDevice3,			// "OK"
	static_cast<TInt>('5'),	// "5"
	static_cast<TInt>('0'),	// "0"
	static_cast<TInt>('*'),	// "*"
	EStdKeyHash,			// "#"
	};

LOCAL_D const TInt32 KTableUidValueSystemEvents[] =
	{
	KNullUidValue,
	KUidChargerStatusValue,
	KUidInboxStatusValue,
	};

CGetPixelsSettingsContainer* CGetPixelsSettingsContainer::NewL(const TRect& aRect)
	{
	CGetPixelsSettingsContainer* self = new(ELeave) CGetPixelsSettingsContainer;
	CleanupStack::PushL(self);
	self->ConstructL(aRect);
	CleanupStack::Pop(self);
	return self;
	}

void CGetPixelsSettingsContainer::ConstructL(const TRect& aRect)
	{
	CreateWindowL();

	for (TInt i = 0; i < iSettingItemLists.Count(); i++)
		{
		LOCAL_D const TInt resourceIds[] =
			{
			R_GETPIXELS_SETTINGITEMLIST_SETTINGS_IMAGE,
			R_GETPIXELS_SETTINGITEMLIST_SETTINGS_FILE,
			R_GETPIXELS_SETTINGITEMLIST_SETTINGS_TRIGGER,
			R_GETPIXELS_SETTINGITEMLIST_SETTINGS_REPEAT,
			R_GETPIXELS_SETTINGITEMLIST_SETTINGS_NOTIFICATION,
			};
		MY_ASSERT(sizeof(resourceIds) / sizeof(resourceIds[0]) == iSettingItemLists.Count());

		iSettingItemLists[i] = new(ELeave) CGetPixelsSettingItemList(*this, iSettings);
		iSettingItemLists[i]->SetMopParent(this);
		iSettingItemLists[i]->ConstructFromResourceL(resourceIds[i]);
		iSettingItemLists[i]->ListBox()->CreateScrollBarFrameL(ETrue);
		iSettingItemLists[i]->ListBox()->ScrollBarFrame()->SetScrollBarVisibilityL(CEikScrollBarFrame::EOff, CEikScrollBarFrame::EAuto);
		iSettingItemLists[i]->MakeVisible(EFalse);
		}

	iCurrentTabIndex = 0;
	iSettingItemLists[iCurrentTabIndex]->MakeVisible(ETrue);

		{
		TUid naviPaneUid;
		naviPaneUid.iUid = EEikStatusPaneUidNavi;

		CEikStatusPane* statusPane = (static_cast<CGetPixelsAppUi*>(iCoeEnv->AppUi()))->StatusPane();
		MY_ASSERT(statusPane != NULL);
		CEikStatusPaneBase::TPaneCapabilities subPane = statusPane->PaneCapabilities(naviPaneUid);

		if (subPane.IsPresent() && subPane.IsAppOwned())
			{
			CAknNavigationControlContainer* naviPane = static_cast<CAknNavigationControlContainer*>(statusPane->ControlL(naviPaneUid));

				{
				TResourceReader reader;
				iCoeEnv->CreateResourceReaderLC(reader, R_GETPIXELS_TABGROUP_SETTINGS);

				iNavigationDecorator = naviPane->CreateTabGroupL(reader);

				CleanupStack::PopAndDestroy();
				}

			naviPane->PushL(*iNavigationDecorator);

			iTabGroup = reinterpret_cast<CAknTabGroup*>(iNavigationDecorator->DecoratedControl());
			MY_ASSERT(iTabGroup != NULL);
			iTabGroup->SetObserver(this);
			}
		}

	SetRect(aRect);
	ActivateL();
	}

CGetPixelsSettingsContainer::CGetPixelsSettingsContainer(void) :
	iAppUi(*reinterpret_cast<CGetPixelsAppUi*>(CCoeEnv::Static()->AppUi()))
	{
	}

CGetPixelsSettingsContainer::~CGetPixelsSettingsContainer()
	{
	for (TInt i = 0; i < iSettingItemLists.Count(); i++)
		{
		delete iSettingItemLists[i];
		}

	if (iNavigationDecorator != NULL)
		{
		iTabGroup->SetObserver(NULL);
		delete iNavigationDecorator;
		}
	}

TInt CGetPixelsSettingsContainer::CountComponentControls(void) const
	{
	return iSettingItemLists.Count();
	}

CCoeControl* CGetPixelsSettingsContainer::ComponentControl(TInt aIndex) const
	{
	if (0 <= aIndex && aIndex < iSettingItemLists.Count())
		{
		return iSettingItemLists[aIndex];
		}

	return NULL;
	}

TKeyResponse CGetPixelsSettingsContainer::OfferKeyEventL(const TKeyEvent& aKeyEvent, TEventCode aType)
	{
	MY_ASSERT(iTabGroup != NULL);
	if (iTabGroup->OfferKeyEventL(aKeyEvent, aType) == EKeyWasConsumed)
		{
		return EKeyWasConsumed;
		}

	MY_ASSERT(iSettingItemLists[iCurrentTabIndex] != NULL);
	if (iSettingItemLists[iCurrentTabIndex]->OfferKeyEventL(aKeyEvent, aType) == EKeyWasConsumed)
		{
		return EKeyWasConsumed;
		}

	return EKeyWasNotConsumed;
	}

void CGetPixelsSettingsContainer::TabChangedL(TInt aIndex)
	{
	iSettingItemLists[iCurrentTabIndex]->MakeVisible(EFalse);

	iCurrentTabIndex = aIndex;

	iSettingItemLists[iCurrentTabIndex]->MakeVisible(ETrue);
	}

void CGetPixelsSettingsContainer::SetActiveTab(TInt aIndex)
	{
	if (iTabGroup != NULL)
		{
		iTabGroup->SetActiveTabByIndex(aIndex);
		TRAPD(ret, TabChangedL(aIndex));
		}
	}

void CGetPixelsSettingsContainer::ChangeSelectedItemL(void)
	{
	MY_ASSERT(iSettingItemLists[iCurrentTabIndex] != NULL);
	iSettingItemLists[iCurrentTabIndex]->EditItemL(iSettingItemLists[iCurrentTabIndex]->ListBox()->CurrentItemIndex(), ETrue);
	}

void CGetPixelsSettingsContainer::ReadSettingsFromModelL(const CGetPixelsSettings& aModel)
	{
	TInt i;

	// Image format
	if (aModel.ImageFormat() == KImageTypePNGUid)
		{
		iSettings.iImageFormat = ESettingsImageFormatPng;
		}
	else if (aModel.ImageFormat() == KImageTypeBMPUid)
		{
		iSettings.iImageFormat = ESettingsImageFormatBmp;
		}
	else if (aModel.ImageFormat() == KImageTypeMBMUid)
		{
		iSettings.iImageFormat = ESettingsImageFormatMbm;
		}
	else if (aModel.ImageFormat() == KImageTypeJPGUid)
		{
		iSettings.iImageFormat = ESettingsImageFormatJpeg;
		}

	// Force threading
	iSettings.iForceThreading = aModel.EnableThreading();

	// Save location
	iSettings.iSaveLocation = (aModel.EnableSaveToExternal()) ?
		CAknMemorySelectionDialog::EMemoryCard : CAknMemorySelectionDialog::EPhoneMemory;

	// Saved directory name
	iSettings.iSavedDirectoryName = aModel.SavedDirectoryName();

	// Saved file name
	iSettings.iSavedFileName = aModel.SavedFileName();

	// Trigger key
	for (i = 0; i < static_cast<TInt>(sizeof(KTableKeyScanCodes) / sizeof(KTableKeyScanCodes[0])); i++)
		{
		if (aModel.TriggerKeyScanCode() == KTableKeyScanCodes[i])
			{
			iSettings.iTriggerKey = i;
			break;
			}
		}

	// Trigger event
	for (i = 0; i < static_cast<TInt>(sizeof(KTableUidValueSystemEvents) / sizeof(KTableUidValueSystemEvents[0])); i++)
		{
		if (aModel.TriggerSystemEvent().iUid == KTableUidValueSystemEvents[i])
			{
			iSettings.iTriggerEvent = i;
			break;
			}
		}

	// Delay time of first shot
	iSettings.iDelayTime = aModel.DelayTime() / 1000;

	// Number of shots
	iSettings.iNumberOfShots = aModel.NumberOfShots();

	// Interval time between shots
	iSettings.iIntervalTime = aModel.IntervalTime() / 1000;

	// Confirmation note
	iSettings.iConfirmationNote = aModel.EnableConfirmationNote();

	// Superimposed text
	iSettings.iSuperimposedText = aModel.EnableSuperimposedText();

	// Vibration notification
	iSettings.iVibrationNotification = aModel.EnableVibrationNotification();

	for (TInt i = 0; i < iSettingItemLists.Count(); i++)
		{
		iSettingItemLists[i]->LoadSettingsL();
		}
	}

void CGetPixelsSettingsContainer::WriteSettingsToModelL(CGetPixelsSettings& aModel) const
	{
	TInt ret;

	for (TInt i = 0; i < iSettingItemLists.Count(); i++)
		{
		iSettingItemLists[i]->StoreSettingsL();
		}

	// Image format
	switch (iSettings.iImageFormat)
		{
	case ESettingsImageFormatPng:
		ret = aModel.SetImageFormat(KImageTypePNGUid);
		MY_ASSERT(ret == KErrNone);
		break;
	case ESettingsImageFormatBmp:
		ret = aModel.SetImageFormat(KImageTypeBMPUid);
		MY_ASSERT(ret == KErrNone);
		break;
	case ESettingsImageFormatMbm:
		ret = aModel.SetImageFormat(KImageTypeMBMUid);
		MY_ASSERT(ret == KErrNone);
		break;
	case ESettingsImageFormatJpeg:
		ret = aModel.SetImageFormat(KImageTypeJPGUid);
		MY_ASSERT(ret == KErrNone);
		break;
	default:
		MY_ASSERT(EFalse);
		break;
		}

	// Force threading
	ret = aModel.SetEnableThreading(iSettings.iForceThreading);
	MY_ASSERT(ret == KErrNone);

	// Save location
	ret = aModel.SetEnableSaveToExternal((iSettings.iSaveLocation != CAknMemorySelectionDialog::EPhoneMemory));
	MY_ASSERT(ret == KErrNone);

	// Saved directory name
	ret = aModel.SetSavedDirectoryName(iSettings.iSavedDirectoryName);
	MY_ASSERT(ret == KErrNone);

	// Saved file name
	ret = aModel.SetSavedFileName(iSettings.iSavedFileName);
	MY_ASSERT(ret == KErrNone);

	// Trigger key
	ret = aModel.SetTriggerKeyScanCode(KTableKeyScanCodes[iSettings.iTriggerKey]);
	MY_ASSERT(ret == KErrNone);

	// Trigger event
	ret = aModel.SetTriggerSystemEvent(TUid::Uid(KTableUidValueSystemEvents[iSettings.iTriggerEvent]));
	MY_ASSERT(ret == KErrNone);

	// Delay time
	ret = aModel.SetDelayTime(iSettings.iDelayTime * 1000);
	MY_ASSERT(ret == KErrNone);

	// Number of shots
	ret = aModel.SetNumberOfShots(iSettings.iNumberOfShots);
	MY_ASSERT(ret == KErrNone);

	// Interval time
	ret = aModel.SetIntervalTime(iSettings.iIntervalTime * 1000);
	MY_ASSERT(ret == KErrNone);

	// Confirmation note
	ret = aModel.SetEnableConfirmationNote(iSettings.iConfirmationNote);
	MY_ASSERT(ret == KErrNone);

	// Superimposed text
	ret = aModel.SetEnableSuperimposedText(iSettings.iSuperimposedText);
	MY_ASSERT(ret == KErrNone);

	// Vibration notification
	ret = aModel.SetEnableVibrationNotification(iSettings.iVibrationNotification);
	MY_ASSERT(ret == KErrNone);
	}

void CGetPixelsSettingsContainer::NotifyEditItemL(TInt aIndex, TBool aCalledFromMenu)
	{
	switch (aIndex)
		{
	case ESettingsImageSetting:
		iAppUi.HandleCommandL(ECommandSwitchToImageSettingsView);
		return;
		/*NOTREACHED*/
		break;

	case ESettingsSavedDirectoryName:
	case ESettingsSavedFileName:
			{
			TFileName tmpFilePath;

			for (;;)
				{
				_LIT(KTmpDriveName, "C:\\");
				_LIT(KTmpExtName, ".tmp");
				TRAPD(ret, GetPixelsUtilities::GetSavedFilePathL(tmpFilePath,
						KTmpDriveName, iSettings.iSavedDirectoryName, iSettings.iSavedFileName, KTmpExtName));
				if (ret == KErrNone && CEikonEnv::Static()->FsSession().IsValidName(tmpFilePath))
					{
					break;
					}

					{
					HBufC* noteText = StringLoader::LoadLC(R_GETPIXELS_TEXT_BAD_FILEPATH);
					CAknInformationNote* note = new(ELeave) CAknInformationNote(ETrue);
					note->ExecuteLD(*noteText);
					CleanupStack::PopAndDestroy(noteText);
					}

				ReadSettingsFromModelL(*iAppUi.GetModel());
				(*iSettingItemLists[iCurrentTabIndex]->SettingItemArray())[aIndex]->LoadL();

				iSettingItemLists[iCurrentTabIndex]->CAknSettingItemList::EditItemL(aIndex, aCalledFromMenu);
				(*iSettingItemLists[iCurrentTabIndex]->SettingItemArray())[aIndex]->StoreL();
				}
			}
		break;

	default:
		break;
		}

	WriteSettingsToModelL(*iAppUi.GetModel());

	switch (aIndex)
		{
	case ESettingsTriggerKey:
		iAppUi.RestartWatchingKeyEventL(iAppUi.GetModel()->TriggerKeyScanCode());
		break;

	case ESettingsTriggerEvent:
		iAppUi.RestartWatchingSystemEventL(iAppUi.GetModel()->TriggerSystemEvent().iUid);
		break;

	default:
		break;
		}
	}

CGetPixelsSettingItemList::CGetPixelsSettingItemList(MGetPixelsSettingItemListObserver& aObserver, TGetPixelsSettings& aSettings) :
	iObserver(aObserver),
	iSettings(aSettings)
	{
	}

void CGetPixelsSettingItemList::SizeChanged(void)
	{
	if (ListBox() != NULL)
		{
		ListBox()->SetRect(Rect());
		}
	}

CAknSettingItem* CGetPixelsSettingItemList::CreateSettingItemL(TInt aIdentifier)
	{
	CAknSettingItem* settingItem = NULL;

	switch (aIdentifier)
		{
	case ESettingsImageFormat:
		settingItem = new(ELeave) CAknEnumeratedTextPopupSettingItem(aIdentifier, iSettings.iImageFormat);
		break;

	case ESettingsImageSetting:
		settingItem = new(ELeave) CAknBigSettingItemBase(aIdentifier);
		break;

	case ESettingsForceThreading:
		settingItem = new(ELeave) CAknBinaryPopupSettingItem(aIdentifier, iSettings.iForceThreading);
		break;

	case ESettingsSaveLocation:
		settingItem = new(ELeave) CAknMemorySelectionSettingItem(aIdentifier, iSettings.iSaveLocation);
		break;

	case ESettingsSavedDirectoryName:
		settingItem = new(ELeave) CAknTextSettingItem(aIdentifier, iSettings.iSavedDirectoryName);
		settingItem->SetSettingPageFlags(settingItem->SettingPageFlags() | CAknTextSettingPage::EZeroLengthAllowed);
		break;

	case ESettingsSavedFileName:
		settingItem = new(ELeave) CAknTextSettingItem(aIdentifier, iSettings.iSavedFileName);
		break;

	case ESettingsTriggerKey:
		settingItem = new(ELeave) CAknEnumeratedTextPopupSettingItem(aIdentifier, iSettings.iTriggerKey);
		break;

	case ESettingsTriggerEvent:
		settingItem = new(ELeave) CAknEnumeratedTextPopupSettingItem(aIdentifier, iSettings.iTriggerEvent);
		break;

	case ESettingsDelayTimeOfFirstShot:
		settingItem = new(ELeave) CAknSliderSettingItemTextLabel(aIdentifier, iSettings.iDelayTime);
		static_cast<CAknSliderSettingItemTextLabel*>(settingItem)->SetTextFormatL(R_GETPIXELS_TEXT_FORMAT_SEC);
		break;

	case ESettingsNumberOfShots:
		settingItem = new(ELeave) CAknIntegerEdwinSettingItem(aIdentifier, iSettings.iNumberOfShots);
		break;

	case ESettingsIntervalTimeBetweenShots:
		settingItem = new(ELeave) CAknSliderSettingItemTextLabel(aIdentifier, iSettings.iIntervalTime);
		static_cast<CAknSliderSettingItemTextLabel*>(settingItem)->SetTextFormatL(R_GETPIXELS_TEXT_FORMAT_SEC);
		break;

	case ESettingsConfirmationNote:
		settingItem = new(ELeave) CAknBinaryPopupSettingItem(aIdentifier, iSettings.iConfirmationNote);
		break;

	case ESettingsSuperimposedText:
		settingItem = new(ELeave) CAknBinaryPopupSettingItem(aIdentifier, iSettings.iSuperimposedText);
		break;

	case ESettingsVibrationNotification:
		settingItem = new(ELeave) CAknBinaryPopupSettingItem(aIdentifier, iSettings.iVibrationNotification);
		break;

	default:
		MY_ASSERT(EFalse);
		break;
		}

	return settingItem;
	}

void CGetPixelsSettingItemList::EditItemL(TInt aIndex, TBool aCalledFromMenu)
	{
	CAknSettingItemList::EditItemL(aIndex, aCalledFromMenu);
	(*SettingItemArray())[aIndex]->StoreL();

	iObserver.NotifyEditItemL((*SettingItemArray())[aIndex]->Identifier(), aCalledFromMenu);
	}
