/**
 *	@file	KeyEventWatcher.cpp
 *	@author	ENDO
 */

#include <e32base.h>
#include <apgwgnam.h>	// for CApaWindowGroupName.

#include "KeyEventWatcher.h"

CKeyEventWatcher* CKeyEventWatcher::NewL(RWindowGroup& aWindowGroup, MKeyEventObserver* aObserver)
	{
	CKeyEventWatcher* self = new(ELeave) CKeyEventWatcher(aWindowGroup, aObserver);
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop(self);

	return self;
	}

CKeyEventWatcher::CKeyEventWatcher(RWindowGroup& aWindowGroup, MKeyEventObserver* aObserver) :
	CActive(EPriorityStandard),
	iObserver(aObserver),
	iId(aWindowGroup.Identifier())
	{
	}

void CKeyEventWatcher::ConstructL(void)
	{
	User::LeaveIfError(iWsSession.Connect());

	iWindowGroup = new(ELeave) RWindowGroup(iWsSession);
	User::LeaveIfError(iWindowGroup->Construct(reinterpret_cast<TUint32>(iWindowGroup), EFalse));

	iWindowGroup->SetOrdinalPosition(-1);
	iWindowGroup->EnableReceiptOfFocus(EFalse);

	iWindowGroupName = CApaWindowGroupName::NewL(iWsSession);
	iWindowGroupName->SetHidden(ETrue);
	iWindowGroupName->SetWindowGroupName(*iWindowGroup);

	CActiveScheduler::Add(this);
	}

CKeyEventWatcher::~CKeyEventWatcher()
	{
	Cancel();

	delete iWindowGroupName;
	if (iWindowGroup != NULL)
		{
		iWindowGroup->Close();
		delete iWindowGroup;
		}
	iWsSession.Close();
	}

void CKeyEventWatcher::StartCaptureKeyL(TUint aKeyCode, TUint aModifierMask, TUint aModifier)
	{
	if (IsActive())
		{
		Cancel();
		}

	User::LeaveIfError(iCaptureKeyHandle = iWindowGroup->CaptureKey(aKeyCode, aModifierMask, aModifier));

	iWsSession.EventReady(&iStatus);
	SetActive();
	}

void CKeyEventWatcher::StartCaptureKeyUpAndDownsL(TUint aScanCode, TUint aModifierMask, TUint aModifier)
	{
	if (IsActive())
		{
		Cancel();
		}

	User::LeaveIfError(iCaptureKeyUpAndDownsHandle = iWindowGroup->CaptureKeyUpAndDowns(aScanCode, aModifierMask, aModifier));

	iWsSession.EventReady(&iStatus);
	SetActive();
	}

void CKeyEventWatcher::RunL(void)
	{
	if (iStatus != KErrNone)
		{
		User::Invariant();
		}

	TWsEvent event;
	iWsSession.GetEvent(event);

	if (iObserver != NULL)
		{
		iObserver->NotifyGlobalKeyEvent(event);
		}
	else
		{
		iWsSession.SendEventToWindowGroup(iId, event);
		}

	TInt foregroundAppId = iWsSession.GetFocusWindowGroup();
	iWsSession.SendEventToWindowGroup(foregroundAppId, event);

	iWsSession.EventReady(&iStatus);
	SetActive();
	}

void CKeyEventWatcher::DoCancel(void)
	{
	iWsSession.EventReadyCancel();

	if (iCaptureKeyHandle != 0UL)
		{
		iWindowGroup->CancelCaptureKey(iCaptureKeyHandle);
		iCaptureKeyHandle = 0UL;
		}
	if (iCaptureKeyUpAndDownsHandle != 0UL)
		{
		iWindowGroup->CancelCaptureKeyUpAndDowns(iCaptureKeyUpAndDownsHandle);
		iCaptureKeyUpAndDownsHandle = 0UL;
		}
	}
