/**
 *	@file	SuperimposedText.cpp
 *	@author	ENDO
 */

#include <e32base.h>
#include <coemain.h>

#include "SuperimposedText.h"

CSuperimposedText* CSuperimposedText::NewL(CCoeEnv& aEnv, const TDesC& aInitialText)
	{
	CSuperimposedText* self = new(ELeave) CSuperimposedText(aEnv, aInitialText);
	CleanupStack::PushL(self);
	self->ConstructL(aEnv, aInitialText);
	CleanupStack::Pop(self);

	return self;
	}

CSuperimposedText::CSuperimposedText(CCoeEnv& aEnv, const TDesC& /* aInitialText */) :
	iEnv(aEnv),
	iWs(aEnv.WsSession()),
	iFont(*aEnv.NormalFont()),
	iSprite(iWs)
	{
	}

void CSuperimposedText::ConstructL(CCoeEnv& /* aEnv */, const TDesC& aInitialText)
	{
	TInt tmpColor, tmpGray;
	iSpriteColorMode = iWs.GetDefModeMaxNumColors(tmpColor, tmpGray);

	const TSize sizeBitmap(iEnv.ScreenDevice()->SizeInPixels().iWidth, iFont.HeightInPixels() + 2);

	iSpriteBitmap = new(ELeave) CFbsBitmap;
	User::LeaveIfError(iSpriteBitmap->Create(sizeBitmap, iSpriteColorMode));
	iSpriteBitmapDevice = CFbsBitmapDevice::NewL(iSpriteBitmap);
	User::LeaveIfError(iSpriteBitmapDevice->CreateContext(iSpriteBitmapGc));

	iMaskBitmap = new(ELeave) CFbsBitmap;
	User::LeaveIfError(iMaskBitmap->Create(sizeBitmap, iSpriteColorMode));
	iMaskBitmapDevice = CFbsBitmapDevice::NewL(iMaskBitmap);
	User::LeaveIfError(iMaskBitmapDevice->CreateContext(iMaskBitmapGc));

	SetTextL(aInitialText);

	iSpriteMember.iBitmap = NULL;
	iSpriteMember.iMaskBitmap = iMaskBitmap;
	iSpriteMember.iInvertMask = ETrue;
	iSpriteMember.iDrawMode = CGraphicsContext::EDrawModePEN;
	iSpriteMember.iOffset = TPoint(0, 0);
	iSpriteMember.iInterval = KMaxTInt32;

	iSpriteMemberDummy.iBitmap = NULL;
	iSpriteMemberDummy.iMaskBitmap = NULL;
	iSpriteMemberDummy.iInvertMask = EFalse;
	iSpriteMemberDummy.iDrawMode = CGraphicsContext::EDrawModePEN;
	iSpriteMemberDummy.iOffset = TPoint(0, 0);
	iSpriteMemberDummy.iInterval = KMaxTInt32;

	ResetL();
	}

CSuperimposedText::~CSuperimposedText()
	{
	iSprite.Close();
	delete iText;
	delete iMaskBitmapGc;
	delete iMaskBitmapDevice;
	delete iMaskBitmap;
	delete iSpriteBitmapGc;
	delete iSpriteBitmapDevice;
	delete iSpriteBitmap;
	}

void CSuperimposedText::SetTextL(const TDesC& aText)
	{
	HBufC* newText = aText.AllocL();
	delete iText;
	iText = newText;

	iSpriteBitmapGc->Reset();
	iSpriteBitmapGc->Clear();

	if (aText.Length() > 0)
		{
		const TInt fontAscent = iFont.AscentInPixels();

		iSpriteBitmapGc->UseFont(&iFont);

		for (TInt i = 0; i < 9; i++)
			{
			const TPoint point(i % 3, i / 3 + fontAscent);
			iSpriteBitmapGc->DrawText(*iText, point);
			}

		iMaskBitmapGc->BitBlt(TPoint(0, 0), iSpriteBitmap);

		iSpriteBitmapGc->SetPenColor(TRgb(255, 255, 255));
		iSpriteBitmapGc->DrawText(*iText, TPoint(1, 1 + fontAscent));

		iSpriteBitmapGc->DiscardFont();
		}
	}

void CSuperimposedText::SetOffset(const TPoint& aPoint)
	{
	iSpriteMember.iOffset = aPoint;
	iSprite.UpdateMember(0, iSpriteMember);
	}

void CSuperimposedText::Show(TTimeIntervalMicroSeconds32 aShowTime)
	{
	iSpriteMember.iBitmap = iSpriteBitmap;
	iSpriteMember.iInterval = aShowTime;
	iSprite.UpdateMember(0, iSpriteMember);
	TRAPD(ret, ResetL());
	}

void CSuperimposedText::Hide(void)
	{
	iSpriteMember.iBitmap = NULL;
	iSprite.UpdateMember(0, iSpriteMember);
	}

void CSuperimposedText::ResetL(void)
	{
	iSprite.Close();
	CleanupClosePushL(iSprite);
	User::LeaveIfError(iSprite.Construct(iEnv.RootWin(), TPoint(0, 0), ESpriteNoChildClip | ESpriteNoShadows));
	User::LeaveIfError(iSprite.AppendMember(iSpriteMember));
	User::LeaveIfError(iSprite.AppendMember(iSpriteMemberDummy));
	User::LeaveIfError(iSprite.Activate());
	CleanupStack::Pop();
	}
