/**
 *	@file	SystemEventWatcher.cpp
 *	@author	ENDO
 */

#include <e32base.h>
#include <e32property.h>	// for RProperty.

#include "SystemEventWatcher.h"

CSystemEventWatcher* CSystemEventWatcher::NewL(MSystemEventObserver& aObserver)
	{
	CSystemEventWatcher* self = new(ELeave) CSystemEventWatcher(aObserver);
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop(self);

	return self;
	}

CSystemEventWatcher::CSystemEventWatcher(MSystemEventObserver& aObserver) :
	CActive(EPriorityStandard),
	iObserver(aObserver)
	{
	}

void CSystemEventWatcher::ConstructL(void)
	{
	CActiveScheduler::Add(this);
	}

CSystemEventWatcher::~CSystemEventWatcher()
	{
	Cancel();
	iProperty.Close();
	}

void CSystemEventWatcher::StartL(TUint aKind)
	{
	iKind = 0U;
	User::LeaveIfError(iProperty.Attach(KUidSystemCategory, aKind));
	iKind = aKind;

	iProperty.Subscribe(iStatus);
	SetActive();
	}

void CSystemEventWatcher::RunL(void)
	{
	TInt value = 0;
	if (iProperty.Get(value) == KErrNone)
		{
		iObserver.NotifySystemEvent(iKind, value);
		}

	iProperty.Subscribe(iStatus);
	SetActive();
	}

void CSystemEventWatcher::DoCancel(void)
	{
	iProperty.Cancel();
	}
